package edu.cmu.paid.authentication;

import java.io.*;
import java.util.*;
import edu.cmu.paid.authentication.Group;
import edu.cmu.paid.authentication.Exception.*;
import edu.cmu.paid.authentication.util.ByteConverterWrapper;

public class TestGroup {

  public static void main(String [] args) {

    Group g1, g2, g3, g4, g5;
    Group g4a,g5a;
    int groupID1, groupID2, groupID3;
    Vector g3access,g5access;
    String grp4S = "Group 4";
    byte [] g4Byte,g5Byte;

    //Instantiate the different groups. 2 and 3 will be preset...
    g1 = new Group();
    g2 = new Group(2,"Group 2");
    g3 = new Group(3,"Group 3");
    g4 = new Group();
    g5 = new Group();

    //Get group IDs...
    groupID1 = g1.getGroupID();
    groupID2 = g2.getGroupID();

    // Output them to the screen if their is an error...
    if (groupID1 != -1){
      System.out.println("Returned inproper groupID for Group 1. Got: "+groupID1+" Expected: -1");
      return;
    }
    if (groupID2 != 2){
      System.out.println("Returned inproper groupID for Group 2. Got: "+groupID2+" Expected: 2");
      return;
    }
    
    // Setting group IDs
    g5.setGroupID(5);
    g4.setGroupID(4);
    g4.setGroupName(grp4S);
    if (g5.getGroupID() != 5){
      System.out.println("Group ID not set to 5 for Group 5. Got: "+g5.getGroupID());
      return;
    }
    
    // Now we test accessible windows...
    g3access = new Vector(6);
    g5access = new Vector(3);
    
    int g3In[] = {3,2,5,1,0,99};
    int g5In[] = {5,1000,52};
    int g4In[] = {3,2,5,1,0,99,1000,52};
    
    for (int i=0; i<6; i++){
      g3access.addElement(new Integer(g3In[i]));
    }
    for (int i=0; i<3; i++){
      g5access.addElement(new Integer(g5In[i]));
    }
    

    try {
      g3.setAccessibleWindows(g3access);
    }
    catch (AccessAlreadyExistsException e) {
      System.out.println("Received AccessAlreadyExitstsException! Not expected");
      return;
    }
    finally {}
    
    try {
      g3.setAccessibleWindows(g5access);
    }
    catch (AccessAlreadyExistsException e) {
      System.out.println("Received the exception we were expecting");
    }

    try {
      g5.setAccessibleWindows(g5access);
      g4.setAccessibleWindows(g5access);
      g4.addAccessibleWindows(g3access);
    }
    catch (AccessAlreadyExistsException e) {
      System.out.println("Received AccessAlreadyExistsException when setting g5 and g4");
      return;
    }

    // Now test to see if gives the proper access
    for (int i=0; i<6; i++){
      if (!g3.hasPermission(g3In[i])){
	System.out.println("Group 3 not given proper access to the applications");
	return;
      }
    }

    for (int i=0; i<3; i++) {
      if (!g5.hasPermission(g5In[i])){
	System.out.println("Group 5 not given proper access to the applications");
	return;
      }
    }

    for (int i=0; i<8 ; i++) {
      if (!g4.hasPermission(g4In[i])){
	System.out.println("Group 4 not given proper access to the applications");
	return;
      }
    }
  
    // Now we do negative tests...
    // First by checking random application IDs
    int badStuff[] = {34,21,111,2333,1010,4,6,7,8,101,91,10};
    for (int i=0; i<badStuff.length;i++){
      if (g4.hasPermission(badStuff[i])){
	System.out.println("g4 has access to "+badStuff[i]);
	return;
      }
      if (g3.hasPermission(badStuff[i])){
	System.out.println("g3 has access to "+badStuff[i]);
	return;
      }
      if (g5.hasPermission(badStuff[i])){
	System.out.println("g5 has access to "+badStuff[i]);
	return;
      }
    }

    System.out.println("Trying to do conversion of objects to byte arrays...");
   
    // First do group 1 the group 2...
    g4Byte = ByteConverterWrapper.toByteArray(g4);
    g5Byte = ByteConverterWrapper.toByteArray(g5);

    System.out.println("This is the byte array 1\n->->-> "+g4Byte+"\n");
    System.out.println("This is the byte array 2\n->->-> "+g5Byte+"\n");

    g4a = (Group) ByteConverterWrapper.toObject(g4Byte);
    g5a = (Group) ByteConverterWrapper.toObject(g5Byte);

    System.out.println("Converted to and from byte[]\nChecking the objects...");

    if (g4a.getGroupID() != g4.getGroupID()) {
      System.out.println("Group 4 was not coverted properly. Got bad GroupID.");
      return;
    }

    if (g5a.getGroupID() != g5.getGroupID()) {
      System.out.println("Group 5 was not converted properly. Got bad GroupID.");
      return;
    }

    Vector convVec = g4.getAccessibleWindows();
    Integer appID;
    for (int i=0; i < convVec.size(); i++){
	appID = (Integer) convVec.elementAt(i);
	  if (!g4a.hasPermission(appID.intValue())) {
	    System.out.println("Reconstructed group 4 does not have Vector of permissions ->->-> "+i);
	  }
      }

    String groupName = g4a.getGroupName();
    if (!groupName.equals(grp4S))
      System.out.println("Group name did not remain after deserilization.");
    else
      System.out.println("Group name is: "+groupName);

    System.out.println("TestGroup passes!");
    return;
  }
  
}
